/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.common.graph;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Set;

import de.tudarmstadt.maki.simonstrator.api.Graphs;

/**
 * A basic class realizing the Graph-Interface
 * 
 * @author Bjoern Richerzhagen
 *
 */
public class BasicGraph implements Graph {

	private Set<DirectedEdge> edges;

	private Set<Node> nodes;

	/**
	 * Creates an empty graph. Components should create new graphs by using the
	 * {@link Graphs} class in the api.
	 */
	public BasicGraph() {
		edges = new LinkedHashSet<>();
		nodes = new LinkedHashSet<>();
	}

	/**
	 * 
	 * @param nodes
	 *            The set of nodes contained in the graph.
	 * @param edges
	 *            Set of edges contained in the graph. Assumes that every start
	 *            and end node in the edges is also contained in the nodes set
	 */
	public BasicGraph(Set<Node> nodes, Set<DirectedEdge> edges) {
		this.edges = edges;
		this.nodes = nodes;

		assert checkGraphConsistency();
	}

	/**
	 * Creates a graph from the given nodes and edges.
	 * 
	 * Duplicate nodes and edges will be ignored
	 */
	public BasicGraph(Collection<Node> nodes, Collection<DirectedEdge> edges) {
		this(new LinkedHashSet<Node>(nodes), new LinkedHashSet<DirectedEdge>(
				edges));
	}

	/**
	 * Creates an edge-induced graph.
	 * 
	 * The nodes are derived from the edges.
	 * 
	 * @param edges
	 *            the edges of the graph
	 */
	public BasicGraph(Collection<DirectedEdge> edges) {
		this(collectEndnodes(edges), edges);
	}

	/**
	 * Creates an edge-induced graph.
	 * 
	 * The nodes are derived from the end nodes of the edges.
	 * 
	 * @param edges
	 *            the edges of the graph
	 */
	@SafeVarargs
	public BasicGraph(DirectedEdge... edges) {
		this(Arrays.asList(edges));
	}

	@Override
	public DirectedEdge createEdge(Node from, Node to) {
		return new DirectedEdge(from, to);
	}

	@Override
	public DirectedEdge createEdge(Node from, Node to, double weight) {
		return new DirectedEdge(from, to, weight);
	}

	@Override
	public Node createNode(Object id) {
		return new Node(id);
	}

	@Override
	public Set<Node> getNodes() {
		return this.nodes;
	}

	@Override
	public Set<DirectedEdge> getEdges() {
		return this.edges;
	}

	/**
	 * Computes subgraph of the currently known local view. This operation
	 * changes the current graph instead of cloning it.
	 */
	@Override
	public BasicGraph getLocalView(Node node, int k) {
		return getLocalView(node, k, false);
	}

	/**
	 * Computes subgraph of the currently known local view. The clone parameter
	 * specifies whether this Graph instance is changed. If clone is true, a new
	 * Graph instance is generated. Elsewise, this Graph instance is modified.
	 */
	@Override
	public BasicGraph getLocalView(Node node, int k, boolean clone) {
		BasicGraph g = clone ? this.clone() : this;

		// filter nodes
		Set<Node> kHopNodes = getNeighbors(node, k, false);

		// determine edges
		Set<DirectedEdge> kHopEdges = new LinkedHashSet<DirectedEdge>();
		for (DirectedEdge edge : g.edges) {
			if (kHopNodes.contains(edge.getStartNode())
					&& kHopNodes.contains(edge.getEndNode())) {
				kHopEdges.add(edge);
			}
		}

		// modify g accordingly
		g.nodes.clear();
		g.nodes.addAll(kHopNodes);
		g.edges.clear();
		g.edges.addAll(kHopEdges);

		assert checkGraphConsistency();

		return g;
	}

	/**
	 * Returns the nodes contained in the k-hop neighborhood of the specified
	 * node. This includes the node itself.
	 * 
	 * If the directedNeighborhood flag is true, edge directions are considered
	 * for the neighborhood
	 */
	@Override
	public Set<Node> getNeighbors(Node node, int k, boolean directedNeighborhood) {
		return getNeighbors(node, k, directedNeighborhood,
				new LinkedHashSet<Node>());
	}

	/**
	 * Checks that every start and end node in the edges is also contained in
	 * the nodes set
	 */
	private boolean checkGraphConsistency() {
		for (DirectedEdge directedEdge : edges) {
			if (!nodes.contains(directedEdge.getStartNode())
					|| !nodes.contains(directedEdge.getEndNode())) {
				return false;
			}
		}

		return true;
	}

	/**
	 * Returns the set of start and end nodes of the given set of edges
	 */
	private static Set<Node> collectEndnodes(Iterable<DirectedEdge> edges) {
		final Set<Node> nodes = new HashSet<>();
		for (final DirectedEdge edge : edges) {
			nodes.add(edge.getStartNode());
			nodes.add(edge.getEndNode());
		}
		return nodes;
	}

	// helper method
	private Set<Node> getNeighbors(Node node, int k,
			boolean directedNeighborhood, LinkedHashSet<Node> visited) {

		assert k >= 0;

		if (k == 0) {
			visited.add(node);
			LinkedHashSet<Node> oneNodeNeighborSet = new LinkedHashSet<Node>();
			oneNodeNeighborSet.add(node);
			return oneNodeNeighborSet;
		}

		visited.add(node);

		LinkedHashSet<Node> neighbors = new LinkedHashSet<Node>();
		neighbors.add(node);

		for (Node neighbor : getNeighbors(node, directedNeighborhood)) {
			if (!visited.contains(neighbor))
				neighbors.addAll(getNeighbors(neighbor, k - 1,
						directedNeighborhood, visited));
		}

		return neighbors;
	}

	/**
	 * For the given node, compute the outgoing edges in the graph
	 * 
	 * @param node
	 *            a node contained in the graph
	 * 
	 * @return
	 */
	@Override
	public Set<DirectedEdge> getOutgoingEdges(Node node) {
		assert this.nodes.contains(node);

		Set<DirectedEdge> outgoingEdges = new LinkedHashSet<DirectedEdge>();
		for (DirectedEdge directedEdge : this.edges) {
			if (directedEdge.getStartNode().equals(node))
				outgoingEdges.add(directedEdge);
		}
		return outgoingEdges;
	}

	/**
	 * For the given node, compute the incoming edges in the graph
	 * 
	 * @param node
	 *            a node contained in the graph
	 * 
	 * @return
	 */
	@Override
	public Set<DirectedEdge> getIncomingEdges(Node node) {
		assert this.nodes.contains(node);

		Set<DirectedEdge> incomingEdges = new LinkedHashSet<DirectedEdge>();
		for (DirectedEdge directedEdge : this.edges) {
			if (directedEdge.getEndNode().equals(node))
				incomingEdges.add(directedEdge);
		}
		return incomingEdges;
	}

	/**
	 * returns the strict 1-hop neighbors of the specified node.
	 * 
	 * If the directedNeighborhood flag is true, edge directions are considered
	 * for the neighborhood
	 * 
	 */
	@Override
	public Set<Node> getNeighbors(Node node, boolean directedNeighborhood) {
		Set<Node> neighbors = new LinkedHashSet<Node>();
		for (DirectedEdge edge : edges) {
			if (edge.getStartNode().equals(node))
				neighbors.add(edge.getEndNode());
			else if (!directedNeighborhood && edge.getEndNode().equals(node))
				neighbors.add(edge.getStartNode());

		}
		return neighbors;
	}

	/**
	 * Returns the strict 1-hop neighbors of the specified node, considering
	 * nodes reachable via directed edges only.
	 * 
	 * @see #getNeighbors(Node, int, boolean)
	 */
	@Override
	public Set<Node> getNeighbors(Node node) {
		return getNeighbors(node, true);
	}

	@Override
	public Set<GraphElement> getAllElements() {
		LinkedHashSet<GraphElement> elements = new LinkedHashSet<GraphElement>();
		elements.addAll(getNodes());
		elements.addAll(getEdges());
		return elements;
	}

	public boolean remove(Node node) {
		return this.nodes.remove(node);
	}

	public boolean remove(DirectedEdge edge) {
		return this.edges.remove(edge);
	}

	@Override
	public boolean contains(GraphElement element) {
		if (element instanceof Node) {
			return getNodes().contains((Node) element);
		} else if (element instanceof DirectedEdge) {
			return getEdges().contains((DirectedEdge) element);
		} else {
			throw new IllegalArgumentException("unknown GraphElement type");
		}
	}

	@Override
	public Graph add(GraphElement element) {
		if (element instanceof Node) {
			nodes.add((Node) element);
		} else if (element instanceof DirectedEdge) {
			edges.add((DirectedEdge) element);
		} else {
			throw new IllegalArgumentException("unknown GraphElement type");
		}
		return this;
	}


	@Override
	public boolean remove(GraphElement element) {
		if (element instanceof Node) {
			return nodes.remove((Node) element);
		} else if (element instanceof DirectedEdge) {
			return edges.remove((DirectedEdge) element);
		} else {
			throw new IllegalArgumentException("unknown GraphElement type");
		}
	}

	@Override
	public BasicGraph clone() {
		// clone nodes
		Set<Node> newNodes = new LinkedHashSet<Node>();
		for (Node node : this.nodes) {
			Node newNode = node.clone();
			newNodes.add(newNode);
		}

		// clone edges
		Set<DirectedEdge> newEdges = new LinkedHashSet<DirectedEdge>();
		for (DirectedEdge directedEdge : this.edges) {
			newEdges.add(directedEdge.clone());
		}

		// return result
		return new BasicGraph(newNodes, newEdges);
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("Nodes: ");
		sb.append(this.nodes);
		sb.append(" Edges: ");
		sb.append(this.edges);
		return sb.toString();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((edges == null) ? 0 : edges.hashCode());
		result = prime * result + ((nodes == null) ? 0 : nodes.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		BasicGraph other = (BasicGraph) obj;
		if (edges == null) {
			if (other.edges != null)
				return false;
		} else if (!edges.equals(other.edges))
			return false;
		if (nodes == null) {
			if (other.nodes != null)
				return false;
		} else if (!nodes.equals(other.nodes))
			return false;
		return true;
	}

	/**
	 * The equals() method checks only equality based on the equal method in the
	 * Node and DirectedEdge implementations. These implementations do not
	 * compare all attributes. This method compares two graphs based on the
	 */
	@Override
	public boolean equalsAttributes(Graph otherGraph) {
		return isContainedInBasedOnEqualAttributes(this.edges,
				otherGraph.getEdges())
				&& isContainedInBasedOnEqualAttributes(otherGraph.getEdges(),
						this.edges)
				&& isContainedInBasedOnEqualAttributes(this.nodes,
						otherGraph.getNodes())
				&& isContainedInBasedOnEqualAttributes(otherGraph.getNodes(),
						this.nodes);

	}

	// returns true if setA is contained in setB based on the
	// GraphelElement.equalsAttributes() method
	private boolean isContainedInBasedOnEqualAttributes(
			Set<? extends GraphElement> setA, Set<? extends GraphElement> setB) {
		for (GraphElement elA : setA) {

			boolean found = false;
			for (GraphElement elB : setB) {
				if (elA.equalsAttributes(elB)) {
					found = true;
					break;
				}
			}

			if (!found)
				return false;

		}
		return true;
	}

	@Override
	public boolean isBidirectional() {
		for (DirectedEdge edge : getEdges()) {

			boolean found = false;
			for (DirectedEdge otherEdge : getEdges()) {

				if (edge.getStartNode().equals(otherEdge.getEndNode())
						&& edge.getEndNode().equals(otherEdge.getStartNode())) {
					found = true;
					break;
				}

			}
			if (!found) {
				return false;
			}
		}

		return true;
	}

	@Override
	public Graph createAndAddNode(Object id) {
		return add(createNode(id));
	}

	@Override
	public Graph createAndAddEdge(Node from, Node to) {
		return add(createEdge(from, to));
	}

	@Override
	public Graph createAndAddEdge(Node from, Node to, double weight) {
		return add(createEdge(from, to, weight));
	}

}
