/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.common.graph;

/**
 * Represents a directed edge in a topology
 * 
 * @author Michael Stein
 */
public class DirectedEdge implements Cloneable, GraphElement {

	private static final double NO_WEIGHT_PLACEHOLDER = Double.NaN;

	private final Node startNode, endNode;

	private final String id;

	private final boolean weighted;

	private final double weight;

	/**
	 * Creates an unweighted, directed edge
	 */
	public DirectedEdge(Node startNode, Node endNode) {
		this(startNode, endNode, false, NO_WEIGHT_PLACEHOLDER, startNode
				.toString() + "->" + endNode.toString());
	}

	/**
	 * Creates an weighted, directed edge
	 */
	public DirectedEdge(Node startNode, Node endNode, double weight) {
		this(startNode, endNode, true, weight, startNode.toString() + "->"
				+ endNode.toString());
	}

	// Generic private constructor
	private DirectedEdge(Node startNode, Node endNode, boolean weighted,
			double weight, String id) {
		this.startNode = startNode;
		this.endNode = endNode;
		this.weight = weight;
		this.weighted = weighted;
		this.id = id;
	}

	public Node getStartNode() {
		return startNode;
	}

	public Node getEndNode() {
		return endNode;
	}

	public boolean isWeighted() {
		return weighted;
	}

	public double getWeight() {
		return weight;
	}

	@Override
	public String getId() {
		return this.id;
	}

	/**
	 * This is an equals() implementation that check equality of all attributes.
	 * This method is not used by hash maps etc.
	 */
	@Override
	public boolean equalsAttributes(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		DirectedEdge other = (DirectedEdge) obj;
		if (endNode == null) {
			if (other.endNode != null)
				return false;
		} else if (!endNode.equals(other.endNode))
			return false;
		if (startNode == null) {
			if (other.startNode != null)
				return false;
		} else if (!startNode.equals(other.startNode))
			return false;
		if (Double.doubleToLongBits(weight) != Double
				.doubleToLongBits(other.weight))
			return false;
		if (weighted != other.weighted)
			return false;
		return true;
	}

	/**
	 * Be careful: This hashCode() implementation considers only the start node
	 * and the end node of the edge. Extent this class if you need different
	 * behavior
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((endNode == null) ? 0 : endNode.hashCode());
		result = prime * result
				+ ((startNode == null) ? 0 : startNode.hashCode());
		return result;
	}

	/**
	 * Be careful: This equals() implementation considers only the start node
	 * and the end node of the edge. Use equalsAttributes for a comparison of
	 * all attributes or extend this class if you need different behavior
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		DirectedEdge other = (DirectedEdge) obj;
		if (endNode == null) {
			if (other.endNode != null)
				return false;
		} else if (!endNode.equals(other.endNode))
			return false;
		if (startNode == null) {
			if (other.startNode != null)
				return false;
		} else if (!startNode.equals(other.startNode))
			return false;
		return true;
	}

	// TODO this implementation of clone creates new instances of Node, even
	// though this is already done in Graph.clone as well. This is still
	// expected to work correctly because hashCode() and equals()
	// are overridden in Node, but it is quite redundant and consumes memory...
	@Override
	public DirectedEdge clone() {
		return new DirectedEdge(startNode.clone(), endNode.clone(),
				weighted, weight, id);
	}

	@Override
	public String toString() {
		return this.getId();
	}
}
