/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.common.graph;

import java.util.Set;

/**
 * Represents a graph structure of an arbitrary topology (updated by BR, divided
 * into a general interface and a basic implementation {@link BasicGraph}).
 * 
 * @author michael.stein, Bjoern Richerzhagen
 * @version 1.0, 02.11.2014
 */
public interface Graph extends Cloneable {

	/*
	 * Creator methods
	 */
	/**
	 * Create and return a node. The node is <strong>not</strong> yet added to
	 * the graph. If a node with the given ID is already in the graph, that node
	 * is returned instead.
	 * 
	 * @param id
	 * @return
	 */
	public Node createNode(Object id);

	/**
	 * Create and return an unweighted edge. The edge is <strong>not</strong>
	 * yet added to the graph. In case the same edge already exists in the
	 * graph, that edge is returned instead.
	 * 
	 * @param from
	 * @param to
	 * @return
	 */
	public DirectedEdge createEdge(Node from, Node to);
	
	/**
	 * Create and return a weighted edge. The edge is <strong>not</strong> yet
	 * added to the graph. In case the same edge already exists in the graph,
	 * that edge is returned instead.
	 * 
	 * @param from
	 * @param to
	 * @param weight
	 *            weight of the edge
	 * @return
	 */
	public DirectedEdge createEdge(Node from, Node to, double weight);

	/*
	 * Graph Interface and convenience methods
	 */

	public Set<Node> getNodes();

	public Set<DirectedEdge> getEdges();

	/**
	 * Computes subgraph of the currently known local view. This operation
	 * changes the current graph instead of cloning it.
	 */
	public BasicGraph getLocalView(Node node, int k);

	/**
	 * Computes subgraph of the currently known local view. The clone parameter
	 * specifies whether this Graph instance is changed. If clone is true, a new
	 * Graph instance is generated. Elsewise, this Graph instance is modified.
	 */
	public BasicGraph getLocalView(Node node, int k, boolean clone);

	/**
	 * For the given node, compute the outgoing edges in the graph
	 * 
	 * @param node
	 *            a node contained in the graph
	 * 
	 * @return
	 */
	public Set<DirectedEdge> getOutgoingEdges(Node node);

	/**
	 * For the given node, compute the incoming edges in the graph
	 * 
	 * @param node
	 *            a node contained in the graph
	 * 
	 * @return
	 */
	public Set<DirectedEdge> getIncomingEdges(Node node);

	/**
	 * Returns the nodes contained in the k-hop neighborhood of the specified
	 * node. This includes the node itself.
	 * 
	 * If the directedNeighborhood flag is true, edge directions are considered
	 * for the neighborhood
	 */
	public Set<Node> getNeighbors(Node node, int k, boolean directedNeighborhood);

	/**
	 * returns the strict 1-hop neighbors of the specified node.
	 * 
	 * If the directedNeighborhood flag is true, edge directions are considered
	 * for the neighborhood
	 * 
	 */
	public Set<Node> getNeighbors(Node node, boolean directedNeighborhood);

	/**
	 * Returns the strict 1-hop neighbors of the specified node, considering
	 * nodes reachable via directed edges only.
	 * 
	 * @see #getNeighbors(Node, int, boolean)
	 */
	public Set<Node> getNeighbors(Node node);

	public Set<GraphElement> getAllElements();

	/**
	 *
	 * TODO define the method's contract
	 * 
	 * @param element
	 * @return the graph object for chaining.
	 */
	public Graph add(GraphElement element);

	/**
	 * Creates and adds the node with the given ID to the graph. If the node is
	 * already present in the graph, the graph remains unchanged
	 * 
	 * @param id
	 * @return
	 */
	public Graph createAndAddNode(Object id);

	public Graph createAndAddEdge(Node from, Node to);

	public Graph createAndAddEdge(Node from, Node to, double weight);

	/**
	 * TODO define the method's contract w.r.t. duplicates etc.
	 * 
	 * @param element
	 * @return
	 */
	public boolean remove(GraphElement element);

	/**
	 * TODO define the method's contract w.r.t. duplicates etc.
	 * 
	 * @param element
	 * @return
	 */
	public boolean contains(GraphElement element);

	/**
	 * Enforce cloning
	 * 
	 * @return
	 */
	public Graph clone();

	/**
	 * The equals() method checks only equality based on the equal method in the
	 * Node and DirectedEdge implementations. These implementations do not
	 * compare all attributes. This method compares two graphs based on the
	 * 
	 * @deprecated should this be part of the general interface?
	 */
	public boolean equalsAttributes(Graph otherGraph);

	/**
	 * Returns true if for each directed edge there exists an edge into the
	 * reverse direction (does not consider edge weight or other attributes)
	 * 
	 * FIXME really slow. was only implemented for debugging purposes and could
	 * be done better for sure... :)
	 * 
	 * @deprecated should this be part of the general interface?
	 */
	public boolean isBidirectional();

}
